Package: ebdm
Type: Package
Title: Estimating Bivariate Dependency from Marginal Data
Version: 3.0.1
Authors@R: 
    c(
      person(given = "Longwen", family = "Shang", role = c("aut"), email = "shanglongwen0918@gmail.com"),
      person(given = "Min", family = "Tsao", role = "aut"),
      person(given = "Xuekui", family = "Zhang", role = c("aut", "cre", "fnd"), email = "ubcxzhang@gmail.com")
    )
Description: Provides statistical methods for estimating bivariate dependency (correlation) from marginal summary statistics across multiple studies. 
    The package supports three modules of bivariate joint distribution estimated from marginal summary data: (1) two binary, (2) two continuous, (3) one binary and one continuous
    These methods enable privacy-preserving joint estimation when individual-level data are unavailable.
    The approaches are detailed in Shang, Tsao, and Zhang (2025a) <doi:10.48550/arXiv.2505.03995> and Shang, Tsao, and Zhang (2025b) <doi:10.48550/arXiv.2508.02057>.
License: GPL (>= 3)
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5.0)
Imports: stats
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-04-21 04:10:01 UTC; shanglongwen
Author: Longwen Shang [aut],
  Min Tsao [aut],
  Xuekui Zhang [aut, cre, fnd]
Maintainer: Xuekui Zhang <ubcxzhang@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-23 13:50:02 UTC
Built: R 4.5.2; ; 2026-04-24 13:21:42 UTC; unix

# Generated by roxygen2: do not edit by hand

export(cor_bin)
export(cor_cont)
export(est_mixture)
importFrom(stats,cov)
importFrom(stats,dnorm)
importFrom(stats,optim)
importFrom(stats,optimize)
importFrom(stats,qchisq)
importFrom(stats,qnorm)
importFrom(stats,sd)

#' Example Data: Mixture Model Summaries
#'
#' Simulated dataset for testing the \code{est_mixture()} function.
#' Each row corresponds to one study providing summary-level data
#' from a two-component normal mixture.
#'
#' @format A data frame with 4 columns:
#' \describe{
#'   \item{ni}{Sample size for each study.}
#'   \item{mi}{Count of group 1 individuals in each study.}
#'   \item{xbar}{Sample mean of the outcome variable.}
#'   \item{s2}{Sample variance of the outcome variable.}
#' }
#' @usage data(mixture_example)
#' @keywords datasets
"mixture_example"

#' Example Data: Binary Variables
#'
#' Simulated dataset for testing the \code{cor_bin()} function.
#'
#' @format A data frame with 3 columns:
#' \describe{
#'   \item{ni}{Sample size per study}
#'   \item{xi}{Count of first binary variable}
#'   \item{yi}{Count of second binary variable}
#' }
#' @usage data(bin_example)
#' @keywords datasets
"bin_example"

#' Estimate Parameters in a Two-Component Gaussian Mixture Using Study-Level Summaries
#' 
#' Estimates group-specific means and standard deviations \eqn{(\mu_1, \mu_0, \sigma_1, \sigma_0)} in a two-component
#' normal mixture model based on aggregate data across multiple studies. The continuous variable \eqn{X}
#' is assumed to follow a Gaussian mixture conditional on a binary group indicator \eqn{Y \in \{0,1\}},
#' with each study reporting only summary-level statistics.
#'
#'#' Two estimation methods are available:
#' \itemize{
#'   \item \strong{"naive"}: Likelihood-based estimator using only sample means.
#'   \item \strong{"gmm"}: Generalized method of moments (GMM) estimator using sample means and variances.
#' }
#'
#' @param ni Integer vector of sample sizes per study.
#' @param xbar Numeric vector of sample means per study.
#' @param s2 Numeric vector of sample variances per study. Required if \code{method = "gmm"}.
#' @param mi Integer vector of group 1 counts per study.
#' @param method Estimation method to use. One of \code{"naive"} or \code{"gmm"}. Default is \code{"gmm"}.
#'
#' @return A named list containing:
#' \describe{
#'   \item{\code{mu1_hat, mu0_hat}}{Estimated means of the two groups.}
#'   \item{\code{sigma1_hat, sigma0_hat}}{Estimated standard deviations.}
#'   \item{\code{se}}{Standard errors of the parameter estimates (NA if \code{method = "naive"}).}
#'   \item{\code{ci}}{List of 95\% confidence intervals for each parameter (NULL if \code{method = "naive"}).}
#'   \item{\code{method}}{A character string indicating the method used.}
#' }
#'
#' @examples
#' # Load example dataset included in the package
#' data(mixture_example)
#'
#' # Estimate using GMM (recommended) with full summary statistics
#' est_mixture(
#'   ni = mixture_example$ni,
#'   xbar = mixture_example$xbar,
#'   s2 = mixture_example$s2,
#'   mi = mixture_example$mi,
#'   method = "gmm"
#' )
#'
#' # Estimate using naive likelihood method (only means used)
#' est_mixture(
#'   ni = mixture_example$ni,
#'   xbar = mixture_example$xbar,
#'   mi = mixture_example$mi,
#'   method = "naive"
#' )
#'
#' @export
#' @importFrom stats optim qnorm sd cov dnorm




est_mixture <- function(ni, xbar, mi, s2=NULL, method = c("gmm", "naive")) {
  
  method    <- match.arg(method)
  
  if (missing(ni) || missing(xbar) || missing(mi))
    stop("`ni`, `xbar`, and `mi` are required.")
  
  if (!is.numeric(ni) || !is.numeric(xbar) || !is.numeric(mi))
    stop("All inputs must be numeric vectors.")
  
  if (length(ni) != length(xbar) || length(ni) != length(mi))
    stop("`ni`, `xbar`, and `mi` must have the same length.")
  
  if (any(mi < 0 | mi > ni))
    stop("All `mi` must satisfy 0 <= mi <= n.")
  
  if (method == "gmm") {
    if (is.null(s2) || any(is.na(s2)))
      stop("`s2` must be provided for method = 'gmm'.")
    
    inputdata <- data.frame(ni = ni, xbar = xbar, s2 = s2, mi = mi, pi_est_individual = mi / ni)
    
  } else {
    inputdata <- data.frame(ni = ni, xbar = xbar, mi = mi, pi_est_individual = mi / ni)
  }
  
  ##############################################################################
  ## Naive estimator (plug-in based)
  ##############################################################################
  baseline_neg_loglik <- function(params, data) {
    mu1 <- params[1]
    mu0 <- params[2]
    log_sigma1 <- params[3]  # work on log scale
    log_sigma0 <- params[4]
    sigma1 <- exp(log_sigma1)
    sigma0 <- exp(log_sigma0)
    
    with(data, {
      pi_i <- pi_est_individual
      mean_i <- pi_i * mu1 + (1 - pi_i) * mu0
      var_i <- (mi / ni^2) * sigma1^2 + ((ni - mi) / ni^2) * sigma0^2
      ll_i <- dnorm(xbar, mean = mean_i, sd = sqrt(var_i), log = TRUE)
      return(-sum(ll_i))
    })
  }
  
  ##############################################################################
  ## GMM Scaled Estimator
  ##############################################################################
  moment_conditions_scale <- function(theta, data) {
    mu1 <- theta[1]
    mu0 <- theta[2]
    sigma1_sq <- theta[3]
    sigma0_sq <- theta[4]
    
    g1_vals <- c()
    g2_vals <- c()
    g3_vals <- c()
    g4_vals <- c()
    g5_vals <- c()
    
    for (i in 1:nrow(data)) {
      ni <- data$ni[i]
      mi <- data$mi[i]
      pi <- mi / ni
      xbar <- data$xbar[i]
      s2 <- data$s2[i]
      
      if (mi <= 1 || mi >= ni - 1) next
      # E[Xbar]
      mu_bar <- pi * mu1 + (1 - pi) * mu0
      g1 <- xbar - mu_bar
      # Var[Xbar]
      var_bar <- sigma1_sq * mi / ni^2 + sigma0_sq * (ni - mi) / ni^2
      if (var_bar <= 0) next
      g3 <- (xbar - mu_bar)^2 - var_bar
      # E[S2]
      E_s2 <- (mi * sigma1_sq + (ni - mi) * sigma0_sq) / ni +
        (mi * (ni - mi)) / (ni * (ni - 1)) * (mu1 - mu0)^2
      # Var[S2]
      nu <- mu1 - mu0
      tau2 <- sigma1_sq / mi + sigma0_sq / (ni - mi)
      var1 <- 2 * (mi - 1) * sigma1_sq^2
      var2 <- 2 * (ni - mi - 1) * sigma0_sq^2
      var3 <- ((mi * (ni - mi)) / ni)^2 * (2 * tau2^2 + 4 * tau2 * nu^2)
      Var_s2 <- (var1 + var2 + var3) / (ni - 1)^2
      if (Var_s2 <= 0) next
      
      g2 <- s2 - E_s2
      g4 <- (s2 - E_s2)^2 - Var_s2
      # Cov[Xbar, S2]
      model_cov <- (2 * mi * (ni - mi)) / (ni^2 * (ni - 1)) * (mu1 - mu0) * (sigma1_sq - sigma0_sq)
      g5 <- (xbar - mu_bar) * (s2 - E_s2) - model_cov
      
      g1_vals <- c(g1_vals, g1)
      g2_vals <- c(g2_vals, g2)
      g3_vals <- c(g3_vals, g3)
      g4_vals <- c(g4_vals, g4)
      g5_vals <- c(g5_vals, g5)
    }
    g_bar <- c(mean(g1_vals), mean(g2_vals), mean(g3_vals), mean(g4_vals), mean(g5_vals))
    sd_g <- apply(cbind(g1_vals, g2_vals, g3_vals, g4_vals, g5_vals), 2, sd, na.rm = TRUE)
    return(list(g_bar = g_bar / sd_g, raw_bar = g_bar, scales = sd_g))
  }
  
  fit_gmm_scale <- function(data, mu1_init, mu0_init, sigma1_init, sigma0_init) {
    gmm_objective <- function(theta, data) {
      moment_result <- moment_conditions_scale(theta, data)
      g <- moment_result$g_bar
      return(sum(g^2))
    }
    theta_init <- c(mu1_init, mu0_init, sigma1_init^2, sigma0_init^2)
    fit <- optim(
      par = theta_init,
      fn = function(p) gmm_objective(p, data),
      method = "L-BFGS-B",
      lower = c(-Inf, -Inf, 1e-6, 1e-6)
    )
    final_moments <- moment_conditions_scale(fit$par, data)
    return(list(
      theta_hat = fit$par,
      mu1_hat = fit$par[1],
      mu0_hat = fit$par[2],
      sigma1_hat = sqrt(fit$par[3]),
      sigma0_hat = sqrt(fit$par[4]),
      converged = fit$convergence == 0,
      objective_value = fit$value,
      scales = final_moments$scales
    ))
  }
  
  ################################################################################
  # SE
  ################################################################################
  compute_moment_matrix <- function(theta, data) {
    mu1 <- theta[1]
    mu0 <- theta[2]
    sigma1_sq <- theta[3]
    sigma0_sq <- theta[4]
    
    moment_matrix <- matrix(NA, nrow = 0, ncol = 5)
    
    for (i in 1:nrow(data)) {
      ni <- data$ni[i]
      mi <- data$mi[i]
      pi <- mi / ni
      xbar <- data$xbar[i]
      s2 <- data$s2[i]
      
      if (mi <= 1 || mi >= ni - 1) next
      
      mu_bar <- pi * mu1 + (1 - pi) * mu0
      g1 <- xbar - mu_bar
      
      var_bar <- sigma1_sq * mi / ni^2 + sigma0_sq * (ni - mi) / ni^2
      if (var_bar <= 0) next
      g3 <- (xbar - mu_bar)^2 - var_bar
      
      # E[S2]
      E_s2 <- (mi * sigma1_sq + (ni - mi) * sigma0_sq) / ni +
        (mi * (ni - mi)) / (ni * (ni - 1)) * (mu1 - mu0)^2
      
      nu <- mu1 - mu0
      tau2 <- sigma1_sq / mi + sigma0_sq / (ni - mi)
      var1 <- 2 * (mi - 1) * sigma1_sq^2
      var2 <- 2 * (ni - mi - 1) * sigma0_sq^2
      var3 <- ((mi * (ni - mi)) / ni)^2 * (2 * tau2^2 + 4 * tau2 * nu^2)
      Var_s2 <- (var1 + var2 + var3) / (ni - 1)^2
      if (Var_s2 <= 0) next
      g2 <- s2 - E_s2
      g4 <- (s2 - E_s2)^2 - Var_s2
      
      model_cov <- (2 * mi * (ni - mi)) / (ni^2 * (ni - 1)) * (mu1 - mu0) * (sigma1_sq - sigma0_sq)
      g5 <- (xbar - mu_bar) * (s2 - E_s2) - model_cov
      
      moment_matrix <- rbind(moment_matrix, c(g1, g2, g3, g4, g5))
      
    }
    
    return(moment_matrix)
  }
  
  compute_jacobian <- function(theta, data, moment_fn, eps = 1e-6) {
    base      <- moment_fn(theta, data)
    scales    <- base$scales                 # 冻住
    g0_scaled <- base$raw_bar / scales
    p <- length(theta); q <- length(g0_scaled)
    J <- matrix(NA_real_, q, p)
    for (j in 1:p) {
      theta_eps      <- theta; theta_eps[j] <- theta_eps[j] + eps
      bump           <- moment_fn(theta_eps, data)
      g_eps_scaled   <- bump$raw_bar / scales  # <- 用同一个 scales
      J[, j]         <- (g_eps_scaled - g0_scaled) / eps
    }
    return(J)
  }
  
  compute_se_gmm <- function(theta_hat, data, moment_fn) {
    
    # 1. 缩放矩条件用“当前 θ̂ 产生的 scales”
    mc_hat        <- moment_fn(theta_hat, data)
    scales_hat    <- mc_hat$scales
    g_mat_raw     <- compute_moment_matrix(theta_hat, data)
    g_mat_scaled  <- sweep(g_mat_raw, 2, scales_hat, "/")
    
    k <- nrow(g_mat_scaled)
    S <- cov(g_mat_scaled, use = "complete.obs") * (k-1)/k
    W <- diag(1, ncol(S))
    
    # 2. Jacobian
    G <- compute_jacobian(theta_hat, data, moment_fn)
    
    A <- t(G) %*% W %*% G
    B <- t(G) %*% W %*% S %*% W %*% G
    V <- solve(A, B) %*% solve(A)     # solve(A) %*% B %*% solve(A)
    
    se <- sqrt(diag(V) / k)
    return(se)
  }
  
  ################################################################################
  # Estimation
  ################################################################################
  
  init_params <- c(mu1 = mean(inputdata$xbar) + 1,
                   mu0 = mean(inputdata$xbar) - 1,
                   log_sigma1 = log(3),
                   log_sigma0 = log(3))
    
  fit_baseline <- tryCatch({
    optim(
      par = init_params,
      fn = baseline_neg_loglik,
      data = inputdata,
      method = "L-BFGS-B",
      lower = c(-Inf, -Inf, log(1e-3), log(1e-3)),
      upper   = c( Inf,  Inf, log(1e+20), log(1e+20))
    )
  }, error = function(e) {
    message("Naive Estimator's optimization failed")
    return(NULL)
  })
  
  # 提取baseline估计值
  par_bl <- fit_baseline$par
  mu1_bl <- par_bl[1]
  mu0_bl <- par_bl[2]
  sigma1_bl <- exp(par_bl[3])
  sigma0_bl <- exp(par_bl[4])
  
  
  
  if (method == "naive") {
    return(list(
      mu1_hat = as.numeric(mu1_bl),
      mu0_hat = as.numeric(mu0_bl),
      sigma1_hat = as.numeric(sigma1_bl),
      sigma0_hat = as.numeric(sigma0_bl),
      se = rep(NA, 4),
      ci = NULL,
      method = "Naive Estimator"
    ))
  }
  
  # 否则继续用GMM估计
  # GMM-scaled
  fit_gmm_scaled <- fit_gmm_scale(
    data = inputdata,
    mu1_init = mu1_bl,
    mu0_init = mu0_bl,
    sigma1_init = sigma1_bl,
    sigma0_init = sigma0_bl
  )
  
  # SE
  se_result <- tryCatch(
    {
      compute_se_gmm(
        theta_hat = fit_gmm_scaled$theta_hat,
        data = inputdata,
        moment_fn = moment_conditions_scale
      )
    },
    error = function(e) {
      message("SE computation failed",": ", e$message)
      return(rep(NA, 4))
    }
  )
  
  
  # GMM
  mu1_hat_scale <- fit_gmm_scaled$theta_hat[1]
  mu0_hat_scale <- fit_gmm_scaled$theta_hat[2]
  sigma1_hat_scale <- sqrt(fit_gmm_scaled$theta_hat[3])
  sigma0_hat_scale <- sqrt(fit_gmm_scaled$theta_hat[4])
  #SE
  mu1_se_scale <- se_result[1]
  mu0_se_scale <- se_result[2]
  sigma1_se_scale <- 0.5 / sigma1_hat_scale * se_result[3]
  sigma0_se_scale <- 0.5 / sigma0_hat_scale * se_result[4]
  se_result <- c(mu1   = mu1_se_scale, mu0   = mu0_se_scale, sigma1= as.numeric(sigma1_se_scale), sigma0= as.numeric(sigma0_se_scale))
  # CI
  ci <- list(
    mu1 = mu1_hat_scale + c(-1, 1) * 1.96 * mu1_se_scale,
    mu0 = mu0_hat_scale + c(-1, 1) * 1.96 * mu0_se_scale,
    sigma1 = sigma1_hat_scale + c(-1, 1) * 1.96 * sigma1_se_scale,
    sigma0 = sigma0_hat_scale + c(-1, 1) * 1.96 * sigma0_se_scale
  )
  
  list(
    mu1_hat = as.numeric(mu1_hat_scale),
    mu0_hat = as.numeric(mu0_hat_scale),
    sigma1_hat = as.numeric(sigma1_hat_scale),
    sigma0_hat = as.numeric(sigma0_hat_scale),
    se = se_result,
    ci = ci,
    method = "GMM Estimator"
  )
}

#' Example Data: Continuous Variables
#'
#' Simulated dataset for testing the \code{cor_cont()} function.
#'
#' @format A data frame with 5 columns:
#' \describe{
#'   \item{Sample_Size}{Sample size for each study.}
#'   \item{Mean_X}{Sample mean of variable X.}
#'   \item{Mean_Y}{Sample mean of variable Y.}
#'   \item{Variance_X}{Sample variance of variable X.}
#'   \item{Variance_Y}{Sample variance of variable Y.}
#' }
#' @usage data(cont_example)
#' @keywords datasets
"cont_example"

#' Estimate the Bivariate Normal Distribution from Marginal Summaries
#'
#' Estimate the correlation coefficient \eqn{\rho} (and marginal means / SDs)
#' of two normally-distributed variables using summary-level data from
#' multiple independent studies.
#'
#' @param n   Numeric vector. Sample size of each study.
#' @param xbar,ybar Numeric vectors. Sample means of the two variables.
#' @param s2x,s2y   Numeric vectors. Sample variances; required for \code{method = "proposed"}.
#' @param method Character. \code{"proposed"} uses the proposed MLE method
#'               in the paper; \code{"weighted"} replicates the weighted
#'               mean based method (Baseline) when no variances are available.
#' @param ci_method Confidence interval type: \code{"none"}, \code{"normal"},
#'                  or \code{"lr"} (likelihood ratio). Only implemented when
#'                  \code{method = "proposed"}.
#' @return A list with elements
#'   \itemize{
#'     \item \code{mu_x, mu_y} : estimated marginal means
#'     \item \code{sigma_x, sigma_y} : estimated SDs
#'     \item \code{rho} : estimated correlation
#'     \item \code{se} : standard error of \code{rho} (proposed only)
#'     \item \code{ci} : confidence interval for \code{rho} (if requested)
#'   }
#' @examples
#' data(cont_example)
#' # Example with full summaries
#' cor_cont(cont_example$Sample_Size, cont_example$Mean_X, cont_example$Mean_Y,
#'  cont_example$Variance_X, cont_example$Variance_Y, method = "proposed", ci_method = "lr")
#'
#' # Only means + n, weighted mean method
#' cor_cont(cont_example$Sample_Size, cont_example$Mean_X, cont_example$Mean_Y, method = "weighted")
#' @export
#' @importFrom stats optim qnorm qchisq
cor_cont <- function(n, xbar, ybar, s2x = NULL, s2y = NULL, method = c("proposed", "weighted"), ci_method = c("none", "normal", "lr")) {
  ##############################################################################
  ## 0.  Check
  method    <- match.arg(method)
  ci_method <- match.arg(ci_method)
  
  if (missing(n))
    stop("`n` (sample sizes) is required.")
  if (!is.numeric(n) || any(n <= 0))
    stop("`n` must be a positive numeric vector.")
  if (!is.numeric(xbar) || !is.numeric(ybar))
    stop("xbar and ybar must be numeric vectors.")
  if (length(n) != length(xbar) || length(xbar) != length(ybar))
    stop("n, xbar, and ybar must have equal length.")
  k <- length(n)
  if (method == "proposed") {
    if (is.null(s2x) || is.null(s2y))
      stop("s2x and s2y are required for method = 'proposed'.")
    if (length(s2x) != k || length(s2y) != k)
      stop("s2x/s2y must be same length as n.")
  }
  
  ##############################################################################
  ## 1. Estimated Means: weighted average
  mu_x <- sum(n * xbar) / sum(n)
  mu_y <- sum(n * ybar) / sum(n)
  
  ##############################################################################
  if (method == "weighted") {
    ##  Weighted Mean Based Method #########################################
    sx2 <- sum(n * (xbar - mu_x)^2) / k
    sy2 <- sum(n * (ybar - mu_y)^2) / k
    rho_num <- sum(n * (xbar - mu_x) * (ybar - mu_y)) / k
    rho_hat <- rho_num / sqrt(sx2 * sy2)
    
    out <- list(mu_x     = mu_x,
                mu_y     = mu_y,
                sigma_x  = sqrt(sx2),
                sigma_y  = sqrt(sy2),
                rho      = rho_hat,
                se       = NA,
                ci       = NULL)
    return(out)
  }
  
  ##############################################################################
  ## 2. Proposed MLE Method  sigma_x, sigma_y
  sx2 <- sum(n * (xbar - mu_x)^2 + (n - 1) * s2x) / sum(n)
  sy2 <- sum(n * (ybar - mu_y)^2 + (n - 1) * s2y) / sum(n)
  sigma_x <- sqrt(sx2)
  sigma_y <- sqrt(sy2)
  ####### Auxiliary functions: numerically stable logarithmic Bessel ###########
  safe_log_bessel <- function(nu, x) {
    log_val <- log(besselI(x, nu, expon.scaled = TRUE)) + x
    if (is.finite(log_val)) {
      return(log_val)
    }
    kappa <- sqrt(nu^2 + x^2)
    eta   <- kappa + nu * log(x / (nu + kappa))
    log_prefactor <- -0.5 * log(2 * pi * kappa)
    eta + log_prefactor
  }
  #######  log likelihood  ###################################################
  loglik <- function(rho) {
    ll <- 0
    for (i in seq_len(k)) {
      
      a <- (n[i] - 4) / 2
      b <- abs((n[i] - 1) * rho * sqrt(s2x[i]) * sqrt(s2y[i]) /
                 ((1 - rho^2) * sigma_x * sigma_y))
      
      ll <- ll +
        log(n[i] / (2 * pi * sigma_x * sigma_y * sqrt(1 - rho^2))) +
        (-n[i] /(2*(1 - rho^2))) * (((xbar[i]-mu_x)^2)/sigma_x^2 - 2*rho*(xbar[i]-mu_x)*(ybar[i]-mu_y)/(sigma_x*sigma_y) + ((ybar[i]-mu_y)^2)/sigma_y^2) +
        ((n[i]-4)/2)*log(s2x[i]*s2y[i]) -
        ((n[i]-1)/(2*(1-rho^2))) * (s2x[i]/sigma_x^2 + s2y[i]/sigma_y^2) -
        (n[i]-1)*log(2*sqrt(sigma_x^2*sigma_y^2*(1-rho^2))/(n[i]-1)) -
        (0.5*log(pi) + lgamma((n[i]-1)/2) + lgamma(n[i]/2 - 1)) +
        0.5*log(pi) + (a+0.5)*(log(2) - log(b)) + lgamma(a+1) + safe_log_bessel(a + 0.5, b)
    }
    ll
  }
  
  ####### Grid Search & L-BFGS-B optimization ##################################
  grid_seq <- seq(-0.99, 0.99, by = 0.001)
  start_rho <- grid_seq[ which.max(sapply(grid_seq, loglik)) ]
  
  opt <- optim(start_rho, loglik,
               method  = "L-BFGS-B",
               lower   = -0.99,
               upper   =  0.99,
               control = list(fnscale = -1))
  
  rho_hat <- opt$par
  
  ## Observed Fisher Info & SE #################################################
  fd2 <- function(f,x,h=1e-5) (f(x+h)-2*f(x)+f(x-h))/h^2
  var_hat <- -1 / fd2(loglik, rho_hat)
  if (!is.finite(var_hat) || var_hat <= 0) {
    warning("Estimated variance of rho is non positive. ",
            "Results may be unreliable due to limited data or numerical issues.")
    se_hat <- NA
  } else {
    se_hat <- sqrt(var_hat)
  }
  
  ###### Confidence Interval ###################################################
  ci <- NULL
  if (ci_method == "normal" && !is.na(se_hat)) {
    z  <- qnorm(0.975)
    ci <- c(lower = max(-1, rho_hat - z*se_hat),
            upper = min( 1, rho_hat + z*se_hat))
  } else if (ci_method == "lr") {
    bound <- -0.5 * qchisq(0.95, 1) + loglik(rho_hat)
    f     <- function(r) abs(loglik(r) - bound)
    low   <- optimize(f, c(-0.99, rho_hat))$minimum
    up    <- optimize(f, c(rho_hat, 0.99))$minimum
    ci    <- c(lower = low, upper = up)
  }
  
  #####  Return  ###################################################
  list(mu_x    = mu_x,
       mu_y    = mu_y,
       sigma_x = sigma_x,
       sigma_y = sigma_y,
       rho     = rho_hat,
       se      = se_hat,
       ci      = ci)
}


#devtools::install()
#devtools::document()
#devtools::check()
#?ebdm_estimate
#data(eg_data)
#ebdm_estimate(eg_data$ni, eg_data$xi, eg_data$yi, ci_method = "lr")
#example(ebdm_estimate)

#' Estimate the Joint Distribution of Two Binary Variables from Marginal Summaries
#'
#' Performs maximum likelihood estimation (MLE) of the joint distribution
#' of two binary variables using only marginal summary data from multiple studies.
#'
#' @param ni Numeric vector. Sample sizes for each dataset.
#' @param xi Numeric vector. Count of observations where variable 1 equals 1.
#' @param yi Numeric vector. Count of observations where variable 2 equals 1.
#' @param ci_method Character string. Method for confidence interval computation.
#'        Options are \code{"none"} (default), \code{"normal"}, or \code{"lr"} (likelihood ratio).
#'
#' @return A named list with point estimates, variance, standard error, and confidence interval (if requested).
#' \describe{
#'   \item{p1_hat}{Estimated marginal probability for variable 1.}
#'   \item{p2_hat}{Estimated marginal probability for variable 2.}
#'   \item{p11_hat}{Estimated joint probability.}
#'   \item{var_hat}{Estimated variance of \code{p11_hat}.}
#'   \item{sd_hat}{Standard error of \code{p11_hat}.}
#'   \item{ci}{Confidence interval for \code{p11_hat}, if requested.}
#' }
#'
#' @examples
#' data(bin_example)
#' cor_bin(bin_example$ni, bin_example$xi, bin_example$yi, ci_method = "lr")
#' @export
#' @importFrom stats optim optimize qnorm qchisq
cor_bin <- function(ni, xi, yi, ci_method = c("none", "normal", "lr")){
  ci_method <- match.arg(ci_method)

  if (!(is.numeric(ni) && is.numeric(xi) && is.numeric(yi))) {
    stop("All inputs must be numeric vectors.")
  }
  if (length(ni) != length(xi) || length(xi) != length(yi)) {
    stop("Input vectors ni, xi, yi must be of equal length.")
  }
  if (anyNA(c(ni, xi, yi))) {
    stop("Input vectors must not contain NA values.")
  }
  if (any(ni %% 1 != 0 | xi %% 1 != 0 | yi %% 1 != 0)) {
    warning("Input counts should be integers; non-integer values will be treated as-is.")
  }

  # Marginal estimates
  sum_ni <- sum(ni)
  sum_xi <- sum(xi)
  sum_yi <- sum(yi)
  p1 <- sum_xi / sum_ni
  p2 <- sum_yi / sum_ni

  data <- data.frame(ni = ni, xi = xi, yi = yi)
  k <- nrow(data)
  ################################################################################
  ################################################################################
  # Log-likelihood term for one z
  tr <- function(z, ni, xi, yi, p1, p2, p11) {
    if (p11 <= 0 || p1 - p11 <= 0 || p2 - p11 <= 0 || 1 - p1 - p2 + p11 <= 0) {
      return(0)
    }
    log_coef <- lfactorial(ni) - lfactorial(z) - lfactorial(xi - z) - lfactorial(yi - z) - lfactorial(ni - xi - yi + z)
    log_prob <- z * log(p11) + (xi - z) * log(p1 - p11) + (yi - z) * log(p2 - p11) + (ni - xi - yi + z) * log(1 - p1 - p2 + p11)
    exp(log_coef + log_prob)
  }
  # Total log-likelihood
  logL <- function(p11) {
    s <- 0
    for (i in 1:k) {
      ni <- data$ni[i]; xi <- data$xi[i]; yi <- data$yi[i]
      z_start <- max(0, xi + yi - ni)
      lim <- min(xi, yi)
      s <- s + log(sum(sapply(z_start:lim, tr, ni = ni, xi = xi, yi = yi, p1 = p1, p2 = p2, p11 = p11)))
    }
    return(s)
  }

  # Observed Fisher Information
  fishinfo <- function(p11) {
    info <- 0
    for (i in 1:k) {
      ni <- data$ni[i]; xi <- data$xi[i]; yi <- data$yi[i]
      z_start <- max(0, xi + yi - ni); lim <- min(xi, yi)
      probs <- sapply(z_start:lim, tr, ni = ni, xi = xi, yi = yi, p1 = p1, p2 = p2, p11 = p11)
      b <- sum(probs)
      d1 <- sapply(z_start:lim, function(z) {
        z/p11 - (xi - z)/(p1 - p11) - (yi - z)/(p2 - p11) +
          (ni - xi - yi + z)/(1 - p1 - p2 + p11)
      })
      d2 <- sapply(z_start:lim, function(z) {
        z/p11^2 + (xi - z)/(p1 - p11)^2 + (yi - z)/(p2 - p11)^2 +
          (ni - xi - yi + z)/(1 - p1 - p2 + p11)^2
      })
      f1 <- sum(probs * (d1^2 - d2))
      f2 <- sum(probs * d1)^2
      info <- info + (b * f1 - f2) / b^2
    }
    return(-1 / info)
  }
  ################################################################################
  ################################################################################
  # Optimize p11
  grid <- which.max(sapply(seq(0.001, min(p1, p2) - 0.001, by = 0.01), logL)) / 100
  opt <- optim(par = grid, fn = logL, method = "L-BFGS-B",
               lower = max(0, p1 + p2 - 1) + 1e-3,
               upper = min(p1, p2) - 1e-3,
               control = list(fnscale = -1))
  p11_hat <- opt$par

  if (logL(grid) > logL(p11_hat)) {
    warning("Numerical optimization result worse than initial grid search value.")
    p11_hat <- grid
  }

  # Standard error and CI
  var_hat <- fishinfo(p11_hat)
  if (var_hat < 0) {
    warning("Estimated variance is negative. The result may be unreliable due to data sparsity or numerical instability.")
    sd_hat <- NA
  } else {
    sd_hat <- sqrt(var_hat)
  }

  ci <- NULL
  if (ci_method == "normal" && !is.na(sd_hat)) {
    z <- qnorm(0.975)
    ci <- c(lower = p11_hat - z * sd_hat, upper = p11_hat + z * sd_hat)
  }
  if (ci_method == "lr") {
    bound <- -0.5 * qchisq(0.95, 1) + logL(p11_hat)
    f <- function(p) abs(logL(p) - bound)
    lwr <- optimize(f, interval = c(max(0, p1 + p2 - 1) + 1e-5, p11_hat))$minimum
    upr <- optimize(f, interval = c(p11_hat, min(p1, p2) - 1e-5))$minimum
    ci <- c(lower = lwr, upper = upr)
  }

  return(list(
    p1_hat = p1,
    p2_hat = p2,
    p11_hat = p11_hat,
    var_hat = var_hat,
    sd_hat = sd_hat,
    ci = ci
  ))
}



#devtools::install()
#devtools::document()
#?ebdm_estimate
#data(eg_data)
#ebdm_estimate(eg_data$ni, eg_data$xi, eg_data$yi, ci_method = "lr")
#example(ebdm_estimate)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor_cont.R
\name{cor_cont}
\alias{cor_cont}
\title{Estimate the Bivariate Normal Distribution from Marginal Summaries}
\usage{
cor_cont(
  n,
  xbar,
  ybar,
  s2x = NULL,
  s2y = NULL,
  method = c("proposed", "weighted"),
  ci_method = c("none", "normal", "lr")
)
}
\arguments{
\item{n}{Numeric vector. Sample size of each study.}

\item{xbar, ybar}{Numeric vectors. Sample means of the two variables.}

\item{s2x, s2y}{Numeric vectors. Sample variances; required for \code{method = "proposed"}.}

\item{method}{Character. \code{"proposed"} uses the proposed MLE method
in the paper; \code{"weighted"} replicates the weighted
mean based method (Baseline) when no variances are available.}

\item{ci_method}{Confidence interval type: \code{"none"}, \code{"normal"},
or \code{"lr"} (likelihood ratio). Only implemented when
\code{method = "proposed"}.}
}
\value{
A list with elements
  \itemize{
    \item \code{mu_x, mu_y} : estimated marginal means
    \item \code{sigma_x, sigma_y} : estimated SDs
    \item \code{rho} : estimated correlation
    \item \code{se} : standard error of \code{rho} (proposed only)
    \item \code{ci} : confidence interval for \code{rho} (if requested)
  }
}
\description{
Estimate the correlation coefficient \eqn{\rho} (and marginal means / SDs)
of two normally-distributed variables using summary-level data from
multiple independent studies.
}
\examples{
data(cont_example)
# Example with full summaries
cor_cont(cont_example$Sample_Size, cont_example$Mean_X, cont_example$Mean_Y,
 cont_example$Variance_X, cont_example$Variance_Y, method = "proposed", ci_method = "lr")

# Only means + n, weighted mean method
cor_cont(cont_example$Sample_Size, cont_example$Mean_X, cont_example$Mean_Y, method = "weighted")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cont_example.R
\docType{data}
\name{cont_example}
\alias{cont_example}
\title{Example Data: Continuous Variables}
\format{
A data frame with 5 columns:
\describe{
  \item{Sample_Size}{Sample size for each study.}
  \item{Mean_X}{Sample mean of variable X.}
  \item{Mean_Y}{Sample mean of variable Y.}
  \item{Variance_X}{Sample variance of variable X.}
  \item{Variance_Y}{Sample variance of variable Y.}
}
}
\usage{
data(cont_example)
}
\description{
Simulated dataset for testing the \code{cor_cont()} function.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cor_bin.R
\name{cor_bin}
\alias{cor_bin}
\title{Estimate the Joint Distribution of Two Binary Variables from Marginal Summaries}
\usage{
cor_bin(ni, xi, yi, ci_method = c("none", "normal", "lr"))
}
\arguments{
\item{ni}{Numeric vector. Sample sizes for each dataset.}

\item{xi}{Numeric vector. Count of observations where variable 1 equals 1.}

\item{yi}{Numeric vector. Count of observations where variable 2 equals 1.}

\item{ci_method}{Character string. Method for confidence interval computation.
Options are \code{"none"} (default), \code{"normal"}, or \code{"lr"} (likelihood ratio).}
}
\value{
A named list with point estimates, variance, standard error, and confidence interval (if requested).
\describe{
  \item{p1_hat}{Estimated marginal probability for variable 1.}
  \item{p2_hat}{Estimated marginal probability for variable 2.}
  \item{p11_hat}{Estimated joint probability.}
  \item{var_hat}{Estimated variance of \code{p11_hat}.}
  \item{sd_hat}{Standard error of \code{p11_hat}.}
  \item{ci}{Confidence interval for \code{p11_hat}, if requested.}
}
}
\description{
Performs maximum likelihood estimation (MLE) of the joint distribution
of two binary variables using only marginal summary data from multiple studies.
}
\examples{
data(bin_example)
cor_bin(bin_example$ni, bin_example$xi, bin_example$yi, ci_method = "lr")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mixture_example.R
\docType{data}
\name{mixture_example}
\alias{mixture_example}
\title{Example Data: Mixture Model Summaries}
\format{
A data frame with 4 columns:
\describe{
  \item{ni}{Sample size for each study.}
  \item{mi}{Count of group 1 individuals in each study.}
  \item{xbar}{Sample mean of the outcome variable.}
  \item{s2}{Sample variance of the outcome variable.}
}
}
\usage{
data(mixture_example)
}
\description{
Simulated dataset for testing the \code{est_mixture()} function.
Each row corresponds to one study providing summary-level data
from a two-component normal mixture.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/est_mixture.R
\name{est_mixture}
\alias{est_mixture}
\title{Estimate Parameters in a Two-Component Gaussian Mixture Using Study-Level Summaries}
\usage{
est_mixture(ni, xbar, mi, s2 = NULL, method = c("gmm", "naive"))
}
\arguments{
\item{ni}{Integer vector of sample sizes per study.}

\item{xbar}{Numeric vector of sample means per study.}

\item{mi}{Integer vector of group 1 counts per study.}

\item{s2}{Numeric vector of sample variances per study. Required if \code{method = "gmm"}.}

\item{method}{Estimation method to use. One of \code{"naive"} or \code{"gmm"}. Default is \code{"gmm"}.}
}
\value{
A named list containing:
\describe{
  \item{\code{mu1_hat, mu0_hat}}{Estimated means of the two groups.}
  \item{\code{sigma1_hat, sigma0_hat}}{Estimated standard deviations.}
  \item{\code{se}}{Standard errors of the parameter estimates (NA if \code{method = "naive"}).}
  \item{\code{ci}}{List of 95\% confidence intervals for each parameter (NULL if \code{method = "naive"}).}
  \item{\code{method}}{A character string indicating the method used.}
}
}
\description{
Estimates group-specific means and standard deviations \eqn{(\mu_1, \mu_0, \sigma_1, \sigma_0)} in a two-component
normal mixture model based on aggregate data across multiple studies. The continuous variable \eqn{X}
is assumed to follow a Gaussian mixture conditional on a binary group indicator \eqn{Y \in \{0,1\}},
with each study reporting only summary-level statistics.
}
\details{
#' Two estimation methods are available:
\itemize{
  \item \strong{"naive"}: Likelihood-based estimator using only sample means.
  \item \strong{"gmm"}: Generalized method of moments (GMM) estimator using sample means and variances.
}
}
\examples{
# Load example dataset included in the package
data(mixture_example)

# Estimate using GMM (recommended) with full summary statistics
est_mixture(
  ni = mixture_example$ni,
  xbar = mixture_example$xbar,
  s2 = mixture_example$s2,
  mi = mixture_example$mi,
  method = "gmm"
)

# Estimate using naive likelihood method (only means used)
est_mixture(
  ni = mixture_example$ni,
  xbar = mixture_example$xbar,
  mi = mixture_example$mi,
  method = "naive"
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bin_example.R
\docType{data}
\name{bin_example}
\alias{bin_example}
\title{Example Data: Binary Variables}
\format{
A data frame with 3 columns:
\describe{
  \item{ni}{Sample size per study}
  \item{xi}{Count of first binary variable}
  \item{yi}{Count of second binary variable}
}
}
\usage{
data(bin_example)
}
\description{
Simulated dataset for testing the \code{cor_bin()} function.
}
\keyword{datasets}
